#=======================================================================
#	$Id: WebServer.pm,v 1.4 2011/01/18 15:41:23 pythontech Exp $
#	Virtual object representing web server context.
#	Copyright (C) 2005-2008  Python Technology Limited
#
#	This program is free software; you can redistribute it and/or
#	modify it under the terms of the GNU General Public License
#	as published by the Free Software Foundation; either version 2
#	of the License, or (at your option) any later version.
#
#	This program is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with this program; if not, write to the Free Software
#	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  
#	02111-1307, USA.
#=======================================================================
package PythonTech::WebServer;
use strict;

#-----------------------------------------------------------------------
#	Configuration parameters:
#	  sessionDb	(optional) Session storage
#			e.g. PythonTech::SessionDB::PHP instance
#	  loginUrl	(optional) URL of page for cookie-based login
#			e.g. PythonTech::Login instance
#-----------------------------------------------------------------------
sub new {
    my $class = shift;
    my $self = {@_};
    $self->{'sessionDb'} ||= delete $self->{'sessiondb'}; # Old name
    $self->{'loginUrl'} ||= delete $self->{'loginurl'};	# Old name
    bless $self, $class;
    return $self;
}

#-----------------------------------------------------------------------
#	Create a CGI query based on current environment
#-----------------------------------------------------------------------
sub cgi_query {
    my($self) = @_;
    my $query = PythonTech::WebServer::Query->new($self);
    return $query;
}

#=======================================================================
#	Query - inherited from standard CGI
#=======================================================================
package PythonTech::WebServer::Query;
use strict;
use CGI;

our @ISA = qw(CGI);

sub new {
    my $class = shift;
    my($server) = @_;
    my $self = new CGI();
    # Assume that CGI is a blessed hash, and that these items are unused
    $self->{'.server'} = $server;
    $self->{'.session'} = undef; # Set on demand
    $self->{'.response'} = new PythonTech::WebServer::Response($self);
    bless $self, $class;
    return $self;
}

#-----------------------------------------------------------------------
#	Get the response
#-----------------------------------------------------------------------
sub response {
    my($self) = @_;
    return $self->{'.response'};
}

#-----------------------------------------------------------------------
#	Get the server object
#-----------------------------------------------------------------------
sub server {
    my($self) = @_;
    return $self->{'.server'};
}

#-----------------------------------------------------------------------
#	Get (and, if necessary, establish) the session of which this
#	query is part.
#-----------------------------------------------------------------------
sub session {
    my($self) = @_;
    unless ($self->{'.session'}) {
	my $sdb = $self->{'.server'}->{'sessionDb'} or
	    die "No sessionDb configured";
	$self->{'.session'} = $sdb->find_session($self) or
	    die ref($sdb)." did not return a session";
    }
    return $self->{'.session'};
}

#-----------------------------------------------------------------------
#	Check that the user has logged in.  If not, redirect to login
#	page
#-----------------------------------------------------------------------
sub require_login {
    my($self, $afterpage) = @_;
    my $user = $self->session->get('userName');
    if (defined $user) {
	return $user;
    } else {
	my $loginurl = $self->{'.server'}->{'loginUrl'};
	die "No login url configured" unless $loginurl;
	# Remember where to come back to after login
	unless (defined $afterpage) {
	    $afterpage = $self->self_url;
	}
	$self->session->set('afterLogin' => $afterpage);
	$self->response->redirect($loginurl);
	return undef;
    }
}

#-----------------------------------------------------------------------
#	Store username in session
#-----------------------------------------------------------------------
sub login {
    my($self, $username) = @_;
    $self->session->set('userName' => $username);
    $self->session->set('loggedIn' => 1);
}

#-----------------------------------------------------------------------
#	Mark as logged out
#-----------------------------------------------------------------------
sub logout {
    my($self) = @_;
    $self->session->clear(['loggedIn']);
}

#-----------------------------------------------------------------------
#	Flush any data that needs to be stored persistently in session
#-----------------------------------------------------------------------
sub flush {
    my($self) = @_;
    if ($self->{'.session'}) {
	$self->{'.session'}->flush;
    }
}

#=======================================================================
#	Response object
#=======================================================================
package PythonTech::WebServer::Response;
use strict;
use PythonTech::Conv qw(html_escape);

sub new {
    my $class = shift;
    my($cgi) = @_;
    my $self = {
	cgi => $cgi,
	status => '200 OK',
	type => 'text/html',
	headers => [],
    };
    bless $self, $class;
    return $self;
}

sub set_status {
    my($self, $status) = @_;
    $self->{'status'} = $status;
}

sub set_type {
    my($self, $type) = @_;
    die "Too late for set_type"
	if $self->{'done_header'};
    $self->{'type'} = $type;
}

sub set_header {
    my($self, $name,$value) = @_;
    die "Too late for set_header"
	if $self->{'done_header'};
    push @{$self->{'headers'}}, $name, $value;
}

sub set_cookie {
    my($self, @args) = @_;
    die "Too late for set_cookie"
	if $self->{'done_header'};
    my $cookie = $self->{'cgi'}->cookie(@args);
    push @{$self->{'cookies'}}, $cookie;
}

#-----------------------------------------------------------------------
#	Write response content, prefixed by header unless already done
#-----------------------------------------------------------------------
sub write {
    my $self = shift;
    unless ($self->{'done_header'}) {
	$self->{'cgi'}->flush;
	print $self->{'cgi'}->header(-status => $self->{'status'},
				     -type => $self->{'type'},
				     -cookie => $self->{'cookies'},
				     @{$self->{'headers'}});
	$self->{'done_header'} = 1;
    }
    print @_;
}

#-----------------------------------------------------------------------
#	Send 204 No Content response
#-----------------------------------------------------------------------
sub no_content {
    my $self = shift;
    $self->set_status('204 No Content');
    $self->write('');
}

#-----------------------------------------------------------------------
#	Send 404 not-found error back to user
#-----------------------------------------------------------------------
sub not_found {
    my($self,$body) = @_;
    $self->set_status('404 Not Found');
    if (defined $body) {
	$self->write($body);
    } else {
	$self->write("<html><head><title>Not Found</title></head>",
		     "<body>The resource ",
		     html_escape($self->{'cgi'}->self_url),
		     " was not found",
		     "</body></html>");
    }
}

#-----------------------------------------------------------------------
#	Send 403 forbidden error back to user
#-----------------------------------------------------------------------
sub forbidden {
    my($self,$body) = @_;
    $self->set_status('403 Forbidden');
    if (defined $body) {
	$self->write($body);
    } else {
	$self->write("<html><head><title>Forbidden</title></head>",
		     "<body>Access to the resource ",
		     html_escape($self->{'cgi'}->self_url),
		     " is forbidden",
		     "</body></html>");
    }
}

#-----------------------------------------------------------------------
#	Send redirection back to user
#-----------------------------------------------------------------------
sub redirect {
    my $self = shift;
    my @args;
    if (@_ == 1) {
	@args = (-uri => $_[0]);
    } else {
	@args = @_;
    }
    unless ($self->{'done_header'}) {
	$self->{'cgi'}->flush;
	print $self->{'cgi'}->redirect(-cookie => $self->{'cookies'}, 
				       @args);
	$self->{'done_header'} = 1;
    }
}

#-----------------------------------------------------------------------
#	After post, 'see other'.
#	N.B. Older browsers e.g. Netscape 4 do not recognise the 303
#	status, so give them a page with an explicit link.
#-----------------------------------------------------------------------
sub see_other {
    my($self, $url,@args) = @_;
    unless ($self->{'done_header'}) {
	$self->{'cgi'}->flush;
	print $self->{'cgi'}->header(-status => '303 See Other',
				     -location => $url,
				     -cookie => $self->{'cookies'},
				     @args);
	my $hurl = &html_escape($url);
	print("<html><head>",
	      "<meta http-equiv=\"Refresh\" content=\"0; URL=$hurl\">",
	      "<title>Now go here</title>",
	      "</head>",
	      "<body>Continue <a href=\"$hurl\">here</a>",
	      "</body></html>");
	$self->{'done_header'} = 1;
    }
}

1;
